// Increase the process priority to prevent input lagging.
import * as os from 'os';
os.setPriority(os.constants.priority.PRIORITY_HIGH);

import { Interception, FilterKeyState, FilterMouseState, KeyboardStroke } from 'node-interception';
import { callDsatApi } from './callDsatApi';

const interception = new Interception();

// Enable the capturing of all strokes.
interception.setFilter("keyboard", FilterKeyState.ALL);
interception.setFilter('mouse', FilterMouseState.NONE);

const SCANCODE_ESC = 0x01;

const normalMap = "  1234567890-=  qwertyuiop[]  asdfghjkl;'~ \\zxcvbnm,./";
const shiftMap = "  !@#$%^&*()_+  QWERTYUIOP{}  ASDFGHJKL:\"` |ZXCVBNM<>?";
function scancode2char(code:number, shift:boolean) {
  return shift?shiftMap[code]:normalMap[code];
}

async function listen() {
  let shift = false;
  let buffer = "";

  while (true) {
    const device = interception.waitSync();
    if (device) {
      const stroke = device.receive();
      if (device.getHardwareId()?.includes("VID_0525&PID_A4AC")) {
        if (!stroke || (stroke?.type === 'keyboard' && stroke.code === SCANCODE_ESC)) break;
  
        if (stroke.state === 1) {
          const keyboardStroke = stroke as KeyboardStroke;
          const code = keyboardStroke.code;
  
          if (code === 42) {
            shift = true;
          } else if (code === 28) {
            await callDsatApi(buffer);
            buffer = "";
          } else {
            const char = scancode2char(code, shift);
            buffer += char;
            shift = false;
          }
        }
      } else if (stroke) {
        // Pass through input for other input devices
        device.send(stroke);
      }
    }
  }

  interception.destroy();
}

try {
  listen();
} catch (error) {
  console.error(error);
}
